from ccapis.interfaces import Bitfinex
from ccapis.interfaces import Bittrex
from ccapis.interfaces import Poloniex
from ccapis.interfaces import Bithumb
from ccapis.interfaces import Korbit
from ccapis.interfaces import Coinone

# from ccapis.interfaces import Coinbase



# Import Built-ins
import logging

# Import Third-Party

# Import Homebrew
from ccapis.formatters.base import Formatter


log = logging.getLogger(__name__)


class BitfinexFormatter(Formatter):

    @staticmethod
    def ticker(data, *args, **kwargs):
        return (data['bid'], data['ask'], data['high'], data['low'], None, None,
                data['last_price'], data['volume'], data['timestamp'])

    @staticmethod
    def order(data, *args, **kwargs):
        try:
            return data['order_id']
        except KeyError:
            return False

    @staticmethod
    def cancel(data, *args, **kwargs):
        return True if 'message' not in data else False

    @staticmethod
    def order_status(data, *args, **kwargs):
        return data['is_live']



# Import Built-ins
import logging

# Import Third-Party

# Import Homebrew
from ccapis.formatters.base import Formatter

log = logging.getLogger(__name__)


class BithumbFormatter(Formatter):

    @staticmethod
    def ticker(data, *args, **kwargs):
        #[bid_price, ask_price, high, low, open, close, last, 24h_vol, ts]
        data = data['data']
        return (data['buy_price'], data['sell_price'], data['max_price'], data['min_price'],
                data['opening_price'], data['closing_price'],data['closing_price'], data['units_traded'], data['date'])

    @staticmethod
    def order(data, *args, **kwargs):
        if data['status'] == '0000':
            return data['order_id']
        else:
            return False

    @staticmethod
    def order_book(data, *args, **kwargs):
        if data['status'] == '0000':
            return data['data']
        else:
            return None

    @staticmethod
    def cancel(data, *args, **kwargs):
        return True if data['status'] == '0000' else False



"""
https://api.bithumb.com
"""

# Import Built-Ins
import logging

# Import Third-Party

# Import  Homebrew
from ccapis.apis.REST import BithumbREST
from ccapis.formatters.base import return_api_response
from ccapis.formatters.bithumb import BithumbFormatter as fmt

# Init Logging Facilities
log = logging.getLogger(__name__)


class Bithumb(BithumbREST):
    def __init__(self, key='', secret='', key_file=''):
        super(Bithumb, self).__init__(key, secret)
        if key_file:
            self.load_key(key_file)

    """
    Bithumb Standardized Methods
    """

    @return_api_response(fmt.ticker)
    def ticker(self, base, counter=None):
        endpoint = 'ticker/'+base
        return self.public_query(endpoint)

    @return_api_response(fmt.order_book)
    #parameter: {'count':10(default 10)}
    def order_book(self, pair, **kwargs):
        orderBookURL = 'orderbook/'+pair
        q = {}
        q.update(kwargs)
        return self.public_query(orderBookURL, params=q)

    # This information is contained in Ticker API
    # @return_api_response(fmt.trades)
    # def trades(self, pair, **kwargs):
    #     q = {'market': pair}
    #     q.update(kwargs)
    #     return self.public_query('getmarkethistory', params=q)

    def _place_order(self, pair, size, price, side, **kwargs):
        q = {'order_currency': pair, 'units': size, 'price': price, 'side': side,
             'Payment_currency':'KRW'}
        q.update(kwargs)
        return self.private_query('trade/place', params=q)

    def _place_order_market(self, pair, size, side, **kwargs):
        q = {'currency': pair, 'units': size}
        q.update(kwargs)
        if side == 'bid':
            return self.private_query('trade/market_buy', params=q)
        else:
            return self.private_query('trade/market_sell', params=q)

    @return_api_response(fmt.order)
    def bid(self, pair, price=0, size=0, market=False, **kwargs):
        if market:
            return self._place_order_market(pair, float(size), 'bid', **kwargs)
        else:
            return self._place_order(pair, float(size), int(price), 'bid', **kwargs)

    @return_api_response(fmt.order)
    def ask(self, pair, price=0, size=0, market=False, **kwargs):
        if market:
            return self._place_order_market(pair, float(size), 'ask', **kwargs)
        else:
            return self._place_order(pair, float(size), int(price), 'ask', **kwargs)

    @return_api_response(fmt.cancel)
    #Parameter: {'order_id': String, 'type': 'bid' or 'ask', 'currency': 'BTC','ETH'...}
    def cancel_order(self, txid, **kwargs):
        q = {'order_id': txid}
        q.update(kwargs)
        return self.private_query('trade/cancel', params=q)

    @return_api_response(fmt.order_status)
    #parameter: {'order_id': String, 'type': 'bid or 'ask', 'currency': 'BTC','ETH'... ,
    #             'count':100(default:100), 'after':Int(Unit TimeStamp)}
    def order(self, txid, **kwargs):
        q = {'order_id': txid}
        q.update(kwargs)
        return self.private_query('info/orders', params=q)

    @return_api_response(fmt.balance)
    # parameter: {'currency': 'BTC' or 'EHT'.. (Default: 'BTC')}
    def balance(self,  **kwargs):
        q = {}
        q.update(kwargs)
        return self.private_query('info/balance')

    @return_api_response(fmt.withdraw)
    # parameter: {'units': float, 'address': String, 'currency': 'BTC', 'ETH'...}
    def withdraw(self, size, tar_addr, **kwargs):
        q = {'units': size, 'address': tar_addr}
        q.update(kwargs)
        return self.private_query('trade/btc_withdrawal', params=q)

    @return_api_response(fmt.deposit)
    def deposit_address(self, pair, **kwargs):
        q = {'currency':pair}
        q.update(kwargs)
        return self.private_query('info/wallet_address', params=kwargs)

    """
    Exchange Specific Methods
    """

    @return_api_response(None)
    def pairs(self):
        return self.public_query('getmarkets')

    @return_api_response(None)
    def currencies(self):
        return self.public_query('getcurrencies')

    @return_api_response(None)
    def statistics(self, pair=None):
        if pair:
            return self.public_query('getmarketsummary', params={'market': pair})
        else:
            return self.public_query('getmarketsummaries')



# Import Built-ins
import logging

# Import Third-Party

# Import Homebrew
from ccapis.formatters.base import Formatter

log = logging.getLogger(__name__)


class BittrexFormatter(Formatter):

    @staticmethod
    def ticker(data, *args, **kwargs):
        data = data['result'][0]
        return (data['Bid'], data['Ask'], data['High'], data['Low'], None, None,
                data['Last'], data['Volume'], data['TimeStamp'])

    @staticmethod
    def order(data, *args, **kwargs):
        if data['success']:
            return data['result']['uuid']
        else:
            return False

    @staticmethod
    def order_book(data, *args, **kwargs):
        if data['success']:
            return data['result']
        else:
            return None

    @staticmethod
    def cancel(data, *args, **kwargs):
        return True if data['success'] else False



"""
Contains all API Client sub-classes, which store exchange specific details
and feature the respective exchanges authentication method (sign()).
"""
# Import Built-ins
import logging
import json
import hashlib
import hmac
import base64

# Import Homebrew
from ccapis.apis.rest import RESTAPIClient


log = logging.getLogger(__name__)


class BitfinexREST(RESTAPIClient):
    def __init__(self, key=None, secret=None, api_version='v1',
                 url='https://api.bitfinex.com', timeout=5):
        super(BitfinexREST, self).__init__(url, api_version=api_version,
                                           key=key, secret=secret,
                                           timeout=timeout)

    def sign(self, url, endpoint, endpoint_path, method_verb, *args, **kwargs):
        try:
            params = kwargs['params']
        except KeyError:
            params = {}
        if self.version == 'v1':
            params['request'] = endpoint_path
            params['nonce'] = self.nonce()

            js = json.dumps(params)
            data = base64.standard_b64encode(js.encode('utf8'))
        else:
            data = '/api/' + endpoint_path + self.nonce() + json.dumps(req)
        h = hmac.new(self.secret.encode('utf8'), data, hashlib.sha384)
        signature = h.hexdigest()
        headers = {
            "X-BFX-APIKEY": self.key,
            "X-BFX-SIGNATURE": signature,
            "X-BFX-PAYLOAD": data
        }
        if self.version == 'v2':
            headers['content-type'] = 'application/json'

        return url, {'headers': headers}

    def public_query(self, endpoint, **kwargs):
        return self.query('GET', endpoint, **kwargs)

    def private_query(self, endpoint, **kwargs):
        return self.query('POST', endpoint, authenticate=True, **kwargs)

    def get_pair(self, base, count):
        return base+count



from ccapis.apis.REST.bitfinex import BitfinexREST
from ccapis.apis.REST.bittrex import BittrexREST
from ccapis.apis.REST.poloniex import PoloniexREST
from ccapis.apis.REST.bithumb import BithumbREST
from ccapis.apis.REST.coinone import CoinoneREST
from ccapis.apis.REST.korbit import KorbitREST
# from ccapis.apis.REST.coinbase import CoinbaseREST



"""
Contains all API Client sub-classes, which store exchange specific details
and feature the respective exchanges authentication method (sign()).
"""
# Import Built-ins
import logging
import hashlib
import hmac
import urllib
import urllib.parse

# Import Homebrew
from ccapis.apis.rest import RESTAPIClient


log = logging.getLogger(__name__)


class PoloniexREST(RESTAPIClient):
    def __init__(self, key=None, secret=None, api_version=None,
                 url='https://poloniex.com', timeout=5):
        super(PoloniexREST, self).__init__(url, api_version=api_version,
                                           key=key, secret=secret,
                                           timeout=timeout)

    def sign(self, uri, endpoint, endpoint_path, method_verb, *args, **kwargs):
        try:
            params = kwargs['params']
        except KeyError:
            params = {}

        params['nonce'] = self.nonce()
        payload = params

        data = urllib.parse.urlencode(payload).encode('utf-8')
        h = hmac.new(self.secret.encode('utf-8'), data, hashlib.sha512)
        signature = h.hexdigest()

        headers = {
            'Key': self.key,
            'Sign': signature
        }

        return uri, {'headers': headers, 'data': params}

    def public_query(self, endpoint, **kwargs):
        return self.query('GET', 'public?command=' + endpoint, **kwargs)

    def private_query(self, endpoint, **kwargs):
        return self.query('POST', endpoint,
                          authenticate=True, **kwargs)

    def get_pair(self, base, count):
        return count + '_' + base



"""
https://api.korbit.co.kr
"""

# Import Built-Ins
import logging

# Import Third-Party

# Import  Homebrew
from ccapis.apis.REST import KorbitREST
from ccapis.formatters.base import return_api_response
from ccapis.formatters.korbit import KorbitFormatter as fmt

# Init Logging Facilities
log = logging.getLogger(__name__)


class Korbit(KorbitREST):
    def __init__(self, key='', secret='', key_file='', **kwargs):
        super(Korbit, self).__init__(key, secret, **kwargs)
        if key_file:
            self.load_key(key_file)

    """
    Coinone Standardized Methods
    """

    @return_api_response(fmt.ticker)
    def ticker(self, base, counter):
        pair = self.get_pair(base, counter)
        q = {'currency_pair': pair}
        return self.public_query('ticker', params=q)

    # @return_api_response(fmt.order_book)
    # #parameter: {'count':10(default 10)}
    # def order_book(self, pair, **kwargs):
    #     orderBookURL = 'orderbook/'+pair
    #     q = {}
    #     q.update(kwargs)
    #     return self.public_query(orderBookURL, params=q)
    #
    # # This information is contained in Ticker API
    # # @return_api_response(fmt.trades)
    # # def trades(self, pair, **kwargs):
    # #     q = {'market': pair}
    # #     q.update(kwargs)
    # #     return self.public_query('getmarkethistory', params=q)
    #
    # def _place_order(self, pair, size, price, side, **kwargs):
    #     q = {'order_currency': pair, 'units': size, 'price': price, 'side': side,
    #          'Payment_currency':'KRW'}
    #     q.update(kwargs)
    #     return self.private_query('trade/place', params=q)
    #
    # def _place_order_market(self, pair, size, side, **kwargs):
    #     q = {'currency': pair, 'units': size}
    #     q.update(kwargs)
    #     if side == 'bid':
    #         return self.private_query('trade/market_buy', params=q)
    #     else:
    #         return self.private_query('trade/market_sell', params=q)
    #
    # @return_api_response(fmt.order)
    # def bid(self, pair, price=0, size=0, market=False, **kwargs):
    #     if market:
    #         return self._place_order_market(pair, float(size), 'bid', **kwargs)
    #     else:
    #         return self._place_order(pair, float(size), int(price), 'bid', **kwargs)
    #
    # @return_api_response(fmt.order)
    # def ask(self, pair, price=0, size=0, market=False, **kwargs):
    #     if market:
    #         return self._place_order_market(pair, float(size), 'ask', **kwargs)
    #     else:
    #         return self._place_order(pair, float(size), int(price), 'ask', **kwargs)
    #
    # @return_api_response(fmt.cancel)
    # #Parameter: {'order_id': String, 'type': 'bid' or 'ask', 'currency': 'BTC','ETH'...}
    # def cancel_order(self, txid, **kwargs):
    #     q = {'order_id': txid}
    #     q.update(kwargs)
    #     return self.private_query('trade/cancel', params=q)
    #
    # @return_api_response(fmt.order_status)
    # #parameter: {'order_id': String, 'type': 'bid or 'ask', 'currency': 'BTC','ETH'... ,
    # #             'count':100(default:100), 'after':Int(Unit TimeStamp)}
    # def order(self, txid, **kwargs):
    #     q = {'order_id': txid}
    #     q.update(kwargs)
    #     return self.private_query('info/orders', params=q)
    #
    # @return_api_response(fmt.balance)
    # # parameter: {'currency': 'BTC' or 'EHT'.. (Default: 'BTC')}
    # def balance(self,  **kwargs):
    #     q = {}
    #     q.update(kwargs)
    #     return self.private_query('info/balance')
    #
    # @return_api_response(fmt.withdraw)
    # # parameter: {'units': float, 'address': String, 'currency': 'BTC', 'ETH'...}
    # def withdraw(self, size, tar_addr, **kwargs):
    #     q = {'units': size, 'address': tar_addr}
    #     q.update(kwargs)
    #     return self.private_query('trade/btc_withdrawal', params=q)
    #
    # @return_api_response(fmt.deposit)
    # def deposit_address(self, pair, **kwargs):
    #     q = {'currency':pair}
    #     q.update(kwargs)
    #     return self.private_query('info/wallet_address', params=kwargs)
    #
    # """
    # Exchange Specific Methods
    # """
    #
    # @return_api_response(None)
    # def pairs(self):
    #     return self.public_query('getmarkets')
    #
    # @return_api_response(None)
    # def currencies(self):
    #     return self.public_query('getcurrencies')
    #
    # @return_api_response(None)
    # def statistics(self, pair=None):
    #     if pair:
    #         return self.public_query('getmarketsummary', params={'market': pair})
    #     else:
    #         return self.public_query('getmarketsummaries')



"""
Contains all API Client sub-classes, which store exchange specific details
and feature the respective exchanges authentication method (sign()).
"""
# Import Built-ins
import logging
import hashlib
import hmac

import urllib
import urllib.parse

# Import Homebrew
from ccapis.apis.rest import RESTAPIClient


log = logging.getLogger(__name__)


class BittrexREST(RESTAPIClient):
    def __init__(self, key=None, secret=None, api_version='v1.1',
                 url='https://bittrex.com/api', timeout=5):
        super(BittrexREST, self).__init__(url, api_version=api_version, key=key,
                                          secret=secret, timeout=timeout)

    def sign(self, url, endpoint, endpoint_path, method_verb, *args, **kwargs):

        try:
            params = kwargs['params']
        except KeyError:
            params = {}

        nonce = self.nonce()

        req_string = endpoint_path + '?apikey=' + self.key + "&nonce=" + nonce + '&'
        req_string += urllib.parse.urlencode(params)

        data = (self.uri + req_string).encode('utf-8')
        h = hmac.new(self.secret.encode('utf-8'), data, hashlib.sha512)
        signature = h.hexdigest()

        headers = {
            "apisign": signature
        }

        return self.uri + req_string, {'headers': headers, 'params': {}}

    def public_query(self, endpoint, **kwargs):
        return self.query('GET', 'public/' + endpoint, **kwargs)

    def private_query(self, endpoint, **kwargs):
        return self.query('GET', endpoint, authenticate=True, **kwargs)

    def get_pair(self, base, count):
        return count + '-' + base



"""
ABC for Exchange APIs
"""
# Import Built-Ins
import logging
import time
from abc import ABCMeta, abstractmethod

# Import Third-Party
from requests import Response
import requests

# Import Homebrew


log = logging.getLogger(__name__)


class RESTAPIResponse(Response):

    def __init__(self, req_response, formatted_json=None):
        for k, v in req_response.__dict__.items():
            self.__dict__[k] = v
        self._formatted = formatted_json

    @property
    def formatted(self):
        return self._formatted

    @formatted.setter
    def formatted(self, value):
        self._formatted = value


class RESTAPIClient(metaclass=ABCMeta):
    """
    Base Class for API ojects. Provides basic methods to interact
    with exchange APIs, such as sending queries and signing messages to pass
    authentication.
    """

    def __init__(self, uri, api_version=None, key=None, secret=None, timeout=5):
        """
        Create API Client object.
        :param uri: string address for api (i.e. https://api.kraken.com/
        :param api_version: version, as required to query an endpoint
        :param key: API access key
        :param secret: API secret
        """
        self.key = key
        self.secret = secret
        self.uri = uri
        self.version = api_version if api_version else ''
        self.timeout = timeout
        log.debug("Initialized API Client for URI: %s; "
                  "Will request on API version: %s" %
                  (self.uri, self.version))

    def load_key(self, path):
        """
        Load key and secret from file.
        :param path: path to file with first two lines are key, secret respectively
        """
        with open(path, 'r') as f:
            self.key = f.readline().strip()
            self.secret = f.readline().strip()

    def nonce(self):
        """
        Creates a Nonce value for signature generation
        :return:
        """
        return str(round(100000 * time.time()) * 2)

    @staticmethod
    def api_request(*args, **kwargs):
        """
        Wrapper which converts a requests.Response into our custom APIResponse
        object
        :param args:
        :param kwargs:
        :return:
        """
        r = requests.request(*args, **kwargs)
        return RESTAPIResponse(r)

    @abstractmethod
    def sign(self, url, endpoint, endpoint_path, method_verb, *args, **kwargs):
        """
        Dummy Signature creation method. Override this in child.
        URL is required to be returned, as some Signatures use the url for
        sig generation, and api calls made must match the address exactly.
        param url: self.uri + self.version + endpoint (i.e https://api.kraken/0/Depth)
        param endpoint: api endpoint to call (i.e. 'Depth')
        param endpoint_path: self.version + endpoint (i.e. '0/Depth')
        param method_verb: valid request type (PUT, GET, POST etc)
        param return:
        """
        url = self.uri

        return url, {'params': {'test_param': "authenticated_chimichanga"}}

    def query(self, method_verb, endpoint, authenticate=False,
              *args, **kwargs):
        """
        Queries exchange using given data. Defaults to unauthenticated query.
        :param method_verb: valid request type (PUT, GET, POST etc)
        :param endpoint: endpoint path for the resource to query, sans the url &
                         API version (i.e. '/btcusd/ticker/').
        :param authenticate: Bool to determine whether or not a signature is
                             required.
        :param args: Optional args for requests.request()
        :param kwargs: Optional Kwargs for self.sign() and requests.request()
        :return: request.response() obj
        """
        if self.version:
            endpoint_path = '/' + self.version + '/' + endpoint
        else:
            endpoint_path = '/' + endpoint

        url = self.uri + endpoint_path
        if authenticate:  # sign off kwargs and url before sending request
            url, request_kwargs = self.sign(url, endpoint, endpoint_path,
                                            method_verb, *args, **kwargs)
        else:
            request_kwargs = kwargs
        log.debug("Making request to: %s, kwargs: %s", url, request_kwargs)
        r = self.api_request(method_verb, url, timeout=self.timeout,
                             **request_kwargs)
        log.debug("Made %s request made to %s, with headers %s and body %s. "
                  "Status code %s", r.request.method,
                  r.request.url, r.request.headers,
                  r.request.body, r.status_code)
        return r




"""
http://docs.bitfinex.com/
"""

# Import Built-Ins
import logging

# Import Third-Party

# Import Homebrew
from ccapis.apis.REST import BitfinexREST
# from bitex.api.WSS.bitfinex import BitfinexWSS
from ccapis.formatters.base import return_api_response
from ccapis.formatters.bitfinex import BitfinexFormatter as fmt

# Init Logging Facilities


log = logging.getLogger(__name__)


class Bitfinex(BitfinexREST):
    def __init__(self, key='', secret='', key_file='', websocket=False):
        super(Bitfinex, self).__init__(key, secret)
        if key_file:
            self.load_key(key_file)
            # if websocket:
            #     self.wss = BitfinexWSS()
            #     self.wss.start()
            # else:
            #     self.wss = None
    """
    CC APIs Standardized Methods
    """

    @return_api_response(fmt.order_book)
    def order_book(self, pair, **kwargs):
        return self.public_query('book/%s' % pair, params=kwargs)

    @return_api_response(fmt.ticker)
    def ticker(self, base, counter, **kwargs):
        pair = self.get_pair(base, counter)
        return self.public_query('pubticker/%s' % pair, params=kwargs)

    @return_api_response(fmt.trades)
    def trades(self, pair, **kwargs):
        return self.public_query('trades/%s' % pair, params=kwargs)

    def _place_order(self, pair, size, price, side, replace, **kwargs):
        q = {'symbol': pair, 'amount': size, 'price': price, 'side': side,
             'type': 'exchange limit'}
        q.update(kwargs)
        if replace:
            return self.private_query('order/cancel/replace', params=q)
        else:
            return self.private_query('order/new', params=q)

    @return_api_response(fmt.order)
    def bid(self, pair, price, size, replace=False, **kwargs):
        return self._place_order(pair, size, price, 'buy', replace=replace,
                                 **kwargs)

    @return_api_response(fmt.order)
    def ask(self, pair, price, size, replace=False, **kwargs):
        return self._place_order(pair, str(size), str(price), 'sell',
                                 replace=replace, **kwargs)

    @return_api_response(fmt.cancel)
    def cancel_order(self, order_id, all=False, **kwargs):

        q = {'order_id': int(order_id)}
        q.update(kwargs)
        if not all:
            return self.private_query('order/cancel', params=q)
        else:
            endpoint = 'order/cancel/all'
            return self.private_query(endpoint)

    @return_api_response(fmt.order_status)
    def order(self, order_id, **kwargs):
        q = {'order_id': order_id}
        q.update(kwargs)
        return self.private_query('order/status', params=q)

    @return_api_response(fmt.balance)
    def balance(self, **kwargs):
        return self.private_query('balances', params=kwargs)

    @return_api_response(fmt.withdraw)
    def withdraw(self, size, tar_addr, **kwargs):
        q = {'withdraw_type': kwargs.pop('withdraw_type'),
             'walletselected': kwargs.pop('walletselected'),
             'amount': size, 'address': tar_addr}
        q.update(kwargs)
        return self.private_query('withdraw', params=q)

    @return_api_response(fmt.deposit)
    def deposit_address(self, **kwargs):
        q = {}
        q.update(kwargs)
        return self.private_query('deposit/new', params=kwargs)

    """
    Exchange Specific Methods
    """

    @return_api_response(None)
    def statistics(self, pair):
        return self.public_query('stats/%s' % pair)

    @return_api_response(None)
    def funding_book(self, currency, **kwargs):
        return self.public_query('lendbook/%s' % currency, params=kwargs)

    @return_api_response(None)
    def lends(self, currency, **kwargs):
        return self.public_query('lends/%s' % currency, params=kwargs)

    @return_api_response(None)
    def pairs(self, details=False):
        if details:
            return self.public_query('symbols_details')
        else:
            return self.public_query('symbols')

    @return_api_response(None)
    def fees(self):
        return self.private_query('account_infos')

    @return_api_response(None)
    def orders(self):
        return self.private_query('orders')

    @return_api_response(None)
    def balance_history(self, currency, **kwargs):
        q = {'currency': currency}
        q.update(kwargs)
        return self.private_query('history/movements', params=q)

    @return_api_response(None)
    def trade_history(self, pair, since, **kwargs):
        q = {'symbol': pair, 'timestamp': since}
        q.update(kwargs)
        return self.private_query('mytrades', params=q)

    @return_api_response(None)
    def positions(self):
        return self.private_query('positions')

    @return_api_response(None)
    def credits(self):
        return self.private_query('credits')

    @return_api_response(None)
    def new_offer(self, currency, amount, rate, period, direction, **kwargs):
        q = {'currency': currency, 'amount': amount, 'rate': rate, 'period': period, 'period': direction}
        q.update(kwargs)
        return self.private_query('offer/new', params=q)

    @return_api_response(None)
    def cancel_offer(self, id, **kwargs):
        q = {'id': id}
        q.update(kwargs)
        return self.private_query('offer/cancel', params=q)



"""
Contains all API Client sub-classes, which store exchange specific details
and feature the respective exchanges authentication method (sign()).
"""
# Import Built-ins
import sys
import time
import math
import base64
import logging
import hashlib
import json
import hmac

# Import Homebrew
from ccapis.apis.rest import RESTAPIClient


log = logging.getLogger(__name__)


class CoinoneREST(RESTAPIClient):
    def __init__(self, key=None, secret=None, api_version='v2',
                 url='https://api.coinone.co.kr', timeout=5, **kwargs):
        if 'user_id' in kwargs:
            self.id = kwargs['user_id']

        super(CoinoneREST, self).__init__(url, api_version=api_version,
                                               key=key, secret=secret,
                                               timeout=timeout)

    def sign(self, url, endpoint, endpoint_path, method_verb, *args, **kwargs):
        __nonce = self.nonce()

        data = {
            'nonce': int(__nonce)
        }
        payload = base64.b64encode(json.dumps(data))

        h = hmac.new(str(self.secret).upper(), str(payload), hashlib.sha512)
        signature = h.hexdigest()

        headers = {
            'Content-type': 'application/json',
            'X-COINONE-PAYLOAD': payload,
            'X-COINONE-SIGNATURE': signature

        }

        return url, {'headers': headers}


    def query(self, method_verb, endpoint, authenticate=False,
              *args, **kwargs):
        """
        Queries exchange using given data. Defaults to unauthenticated query.
        :param method_verb: valid request type (PUT, GET, POST etc)
        :param endpoint: endpoint path for the resource to query, sans the url &
                         API version (i.e. '/btcusd/ticker/').
        :param authenticate: Bool to determine whether or not a signature is
                             required.
        :param args: Optional args for requests.request()
        :param kwargs: Optional Kwargs for self.sign() and requests.request()
        :return: request.response() obj
        """

        endpoint_path = '/' + endpoint

        url = self.uri + endpoint_path
        if authenticate:  # sign off kwargs and url before sending request
            endpoint_path = '/' + self.version + '/' + endpoint
            url, request_kwargs = self.sign(url, endpoint, endpoint_path,
                                            method_verb, *args, **kwargs)
        else:
            request_kwargs = kwargs
        log.debug("Making request to: %s, kwargs: %s", url, request_kwargs)
        r = self.api_request(method_verb, url, timeout=self.timeout,
                             **request_kwargs)
        log.debug("Made %s request made to %s, with headers %s and body %s. "
                  "Status code %s", r.request.method,
                  r.request.url, r.request.headers,
                  r.request.body, r.status_code)
        return r


    def public_query(self, endpoint, **kwargs):
        return self.query('GET', endpoint, **kwargs)

    def private_query(self, endpoint, **kwargs):
        return self.query('GET', endpoint, authenticate=True, **kwargs)



from ccapis.interfaces.bitfinex import Bitfinex
from ccapis.interfaces.bittrex import Bittrex
from ccapis.interfaces.poloniex import Poloniex
from ccapis.interfaces.bithumb import Bithumb
from ccapis.interfaces.coinone import Coinone
from ccapis.interfaces.korbit import Korbit
# from ccapis.interfaces.coinbase import Coinbase



"""
https://poloniex.com/support/api/
"""

# Import Built-Ins
import logging

# Import Third-Party

# Import Homebrew
from ccapis.apis.REST import PoloniexREST
# from bitex.api.WSS.poloniex import PoloniexWSS
from ccapis.formatters.base import return_api_response
from ccapis.formatters.poloniex import PoloniexFormatter as fmt
# Init Logging Facilities


log = logging.getLogger(__name__)


class Poloniex(PoloniexREST):
    def __init__(self, key='', secret='', key_file='', websocket=False):
        super(Poloniex, self).__init__(key, secret)
        if key_file:
            self.load_key(key_file)
        # if websocket:
        #     self.wss = PoloniexWSS()
        #     self.wss.start()
        # else:
        #     self.wss = None

    """
    CC APIs Standardized Methods
    """

    @return_api_response(fmt.ticker)
    def ticker(self, base, counter, **kwargs):
        # pair = self.get_pair(base, counter)
        # return self.public_query('returnTicker', params=kwargs)[pair]
        return self.public_query('returnTicker', params=kwargs)

    @return_api_response(fmt.order_book)
    def order_book(self, pair, **kwargs):
        kwargs['currencyPair'] = pair
        return self.public_query('returnOrderBook', params=kwargs)

    @return_api_response(fmt.trades)
    def trades(self, pair, **kwargs):
        kwargs['currencyPair'] = pair
        return self.public_query('returnTradeHistory', params=kwargs)

    @return_api_response(fmt.order)
    def bid(self, pair, rate, size, **kwargs):
        q = {'command': 'buy', 'currencyPair': pair, 'amount': size,
             'rate': rate}
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)

    @return_api_response(fmt.order)
    def ask(self, pair, rate, size, **kwargs):
        q = {'command': 'sell', 'currencyPair': pair, 'amount': size,
             'rate':    rate}
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)

    @return_api_response(fmt.cancel)
    def cancel_order(self, txid, **kwargs):
        q = {'orderNumber': txid, 'command': 'cancelOrder'}
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)

    @return_api_response(fmt.order_status)
    def order(self, order_id, **kwargs):
        raise NotImplementedError()

    @return_api_response(fmt.balance)
    def balance(self, detailed=False, **kwargs):
        q = {}
        if detailed:
            q['command'] = 'returnCompleteBalances'
        else:
            q['command'] = 'returnBalances'
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)

    @return_api_response(fmt.withdraw)
    def withdraw(self, size, tar_addr, **kwargs):
        q = {'currency': kwargs.pop('currency'), 'amount': size, 'address': tar_addr}
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)

    @return_api_response(fmt.deposit)
    def deposit_address(self, currency, **kwargs):
        q = {'command': 'returnDepositAddresses'}
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)


    """
    Exchange Specific Methods
    """

    @return_api_response(None)
    def currencies(self):
        return self.public_query('returnCurrencies')

    @return_api_response(None)
    def hloc(self, pair, **kwargs):
        kwargs['currencyPair'] = pair
        return self.public_query('returnChartData')


    @return_api_response(None)
    def balance_history(self, **kwargs):
        q = {'command': 'returnDepositsWithdrawals'}
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)

    @return_api_response(None)
    def orders(self, pair='all', **kwargs):
        q = {'command': 'returnOpenOrders', 'currencyPair': pair}
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)

    @return_api_response(None)
    def trade_history(self, pair='all', **kwargs):
        q = {'currencyPair': pair, 'command': 'returnTradeHistory'}
        q.update(kwargs)
        return self.private_query('tradingApi', params=q)

    @return_api_response(None)
    def update_order(self, txid, rate, **kwargs):
        q = {'command': 'moveOrder', 'rate': rate, 'orderNumber': txid}
        q.update(kwargs)
        return self.query('tradingApi', params=q)

    @return_api_response(None)
    def fees(self):
        return self.private_query('tradingApi',
                                  params={'command': 'returnFeeInfo'})



# Import Built-Ins
import logging
from queue import Queue, Empty
from threading import Thread

# Import Third-Party

# Import Homebrew

# Init Logging Facilities
log = logging.getLogger(__name__)


class WSSAPIClient:
    """
    Base Class with no actual connection functionality. This is added in the
    subclass, as the various wss APIs are too diverse in order to distill a
    sensible pool of common attributes.
    """
    def __init__(self, addr, name):
        """
        Initialize Object.
        :param addr:
        :param name:
        """
        log.debug("WSSAPIClient.__init__(): Initializing Websocket API")
        self.addr = addr
        self.name = name
        self.running = False

        # External Interface to interact with WSS.
        self.interface = Queue()

        # Internal Command Queue for restarts, stops and starts
        self._controller_q = Queue()

        # Queue storing all received data
        self.data_q = Queue()

        # Internal Controller thread, responsible for starts / restarts / stops
        self._controller_thread = None

    def start(self):
        """
        Starts threads. Extend this in your child class.
        :return:
        """
        log.info("WSSAPIClient.start(): Starting Basic Facilities")
        self.running = True
        if self._controller_thread is None or not self._controller_thread.is_alive():
            self._controller_thread = Thread(target=self._controller,
                                             daemon=True,
                                             name='%s Controller Thread' %
                                                  self.name)
            self._controller_thread.start()

    def stop(self):
        """
        Stops Threads. Overwrite this in your child class as necessary.
        :return:
        """
        log.debug("WSSAPIClient.stop(): Stopping..")
        self.running = False

    def restart(self):
        """
        Restart Threads.
        :return:
        """
        log.debug("WSSAPIClient.restart(): Restarting API Client..")
        self.stop()
        self.start()

    def _controller(self):
        """
        This method runs in a dedicated thread, calling self.eval_command().
        :return:
        """
        while self.running:
            try:
                cmd = self._controller_q.get(timeout=1)
            except (TimeoutError, Empty):
                continue
            log.debug("WSSAPIClient._controller(): Received command: %s", cmd)
            Thread(target=self.eval_command, args=(cmd,)).start()

    def send(self, payload):
        """
        Method to send instructions for subcribing, unsubscribing, etc to
        the exchange API.
        :return:
        """
        raise NotImplementedError()

    def eval_command(self, cmd):
        """
        Evaluates commands issued by internal threads. Extend this as necessary.
        :param cmd:
        :return:
        """
        if cmd == 'restart':
            self.restart()

        elif cmd == 'stop':
            self.stop()

        else:
            raise ValueError("Unknown Command passed to controller! %s" % cmd)

    def get(self, **kwargs):
        return self.data_q.get(**kwargs)



"""
Contains all API Client sub-classes, which store exchange specific details
and feature the respective exchanges authentication method (sign()).
"""
# Import Built-ins
import sys
import time
import math
import base64
import logging
import hashlib
import hmac
import urllib
import datetime

# Import Homebrew
from ccapis.apis.rest import RESTAPIClient


log = logging.getLogger(__name__)


class KorbitREST(RESTAPIClient):
    def __init__(self, key=None, secret=None, api_version='v1',
                 url='https://api.korbit.co.kr', timeout=5, **kwargs):
        if 'user_id' in kwargs:
            self.id = kwargs['user_id']

        if 'password' in kwargs:
            self.password = kwargs['password']

        self.expire = 3600
        self.access_token = ''
        self.refresh_token = ''
        self.access_time = 0
        super(KorbitREST, self).__init__(url, api_version=api_version,
                                           key=key, secret=secret,
                                           timeout=timeout)

    def authentication(self):
        params = {
            'client_id': self.key,
            'client_secret': self.secret,
            'username': self.id,
            'passworkd': self.password,
            'grant_type': 'password'
        }

        data =  urllib.parse.urlencode(params)

        self.access_time = datetime.datetime.now()

        return self.api_request('POST', 'https://api.korbit.co.kr/v1/oauth2/access_token', data=data)

    def reauthentication(self):
        params = {
            'client_id': self.key,
            'client_secret': self.secret,
            'refresh_token': self.refresh_token,
            'grant_type': 'refresh_token'
        }

        data =  urllib.parse.urlencode(params)

        return self.api_request('POST', 'https://api.korbit.co.kr/v1/oauth2/access_token', data=data)

    def sign(self, url, endpoint, endpoint_path, method_verb, *args, **kwargs):
        __nonce = self.nonce()
        try:
            params = kwargs['params']
        except KeyError:
            params = {}

        params['nonce'] = __nonce
        current_time = datetime.datetime.now()
        if self.access_time is 0 :
            response = self.authentication()

            self.expire = response['expires_in']
            __token_type = response['token_type']
            self.refresh_token = response['refresh_token']
            self.access_token = response['access_token']

        else :
            time_delta = current_time - self.access_time
            if time_delta.total_seconds()-600 > self.expire:
                response = self.reauthentication()

                self.expire = response['expires_in']
                __token_type = response['token_type']
                self.refresh_token = response['refresh_token']
                self.access_token = response['access_token']

        token = __token_type + chr(0) + self.access_token
        headers = {
            'Authorization': token
        }

        return url, {'headers': headers}

    def public_query(self, endpoint, **kwargs):
        return self.query('GET', endpoint, **kwargs)

    def private_query(self, endpoint, **kwargs):
        return self.query('GET', endpoint, authenticate=True, **kwargs)

    def get_pair(self, base, count):
        base = base.lower()
        count = count.lower()
        return base + '_' + count



"""
https://bittrex.com/Home/Api
"""

# Import Built-Ins
import logging

# Import Third-Party

# Import Homebrew
from ccapis.apis.REST import BittrexREST
from ccapis.formatters.base import return_api_response
from ccapis.formatters.bittrex import BittrexFormatter as fmt
# Init Logging Facilities


log = logging.getLogger(__name__)


class Bittrex(BittrexREST):
    def __init__(self, key='', secret='', key_file=''):
        super(Bittrex, self).__init__(key, secret)
        if key_file:
            self.load_key(key_file)

    """
    CC APIs Standardized Methods
    """

    @return_api_response(fmt.ticker)
    def ticker(self, base, counter):
        pair = self.get_pair(base, counter)
        q = {'market': pair}
        return self.public_query('getmarketsummary', params=q)

    @return_api_response(fmt.order_book)
    def order_book(self, pair, side='both', **kwargs):
        q = {'market': pair, 'type': side}
        q.update(kwargs)
        return self.public_query('getorderbook', params=q)

    @return_api_response(fmt.trades)
    def trades(self, pair, **kwargs):
        q = {'market': pair}
        q.update(kwargs)
        return self.public_query('getmarkethistory', params=q)

    @return_api_response(fmt.order)
    def bid(self, pair, price, vol, market=False, **kwargs):
        q = {'market': pair, 'rate': price, 'quantity': vol}
        q.update(kwargs)
        if market:
            # send market order
            return self.private_query('market/buymarket', params=q)
        else:
            # send limit order
            return self.private_query('market/buylimit', params=q)

    @return_api_response(fmt.order)
    def ask(self, pair, price, vol, market=False, **kwargs):
        q = {'market': pair, 'rate': price, 'quantity': vol}
        q.update(kwargs)
        if market:
            # send market order
            return self.private_query('market/sellmarket', params=q)
        else:
            # send limit order
            return self.private_query('market/selllimit', params=q)

    @return_api_response(fmt.cancel)
    def cancel_order(self, txid):
        q = {'uuid': txid}
        return self.private_query('market/cancel', params=q)

    @return_api_response(fmt.order_status)
    def order(self, order_id, **kwargs):
        q = {'uuid': order_id}
        q.update(kwargs)
        return self.private_query('account/getorder', params=q)

    @return_api_response(fmt.balance)
    def balance(self):
        return self.private_query('account/getbalances')

    @return_api_response(fmt.withdraw)
    def withdraw(self, size, tar_addr, **kwargs):
        q = {'quantity': size, 'address': tar_addr}
        q.update(kwargs)
        return self.private_query('account/withdraw', params=q)

    @return_api_response(fmt.deposit)
    def deposit_address(self, **kwargs):
        return self.private_query('account/getdepositaddress', params=kwargs)

    """
    Exchange Specific Methods
    """

    @return_api_response(None)
    def pairs(self):
        return self.public_query('getmarkets')

    @return_api_response(None)
    def currencies(self):
        return self.public_query('getcurrencies')

    @return_api_response(None)
    def statistics(self, pair=None):
        if pair:
            return self.public_query('getmarketsummary', params={'market': pair})
        else:
            return self.public_query('getmarketsummaries')



from distutils.core import setup


setup(name='ccapis', version='0.3.4', author='Jonathan',
      author_email='imyme6yo@gmail.com',
      url="https://github.com/paragonjh/ccapis.git",
      packages=['ccapis', 'ccapis.apis', 'ccapis.apis.WSS', 'ccapis.apis.REST',
      #packages=['ccapis', 'ccapis.apis', 'ccapis.apis.REST',
                'ccapis.interfaces', 'ccapis.formatters'],
      install_requires=['requests', 'autobahn', 'pusherclient'],
      description='Python3-based API Framework for Crypto Exchanges',
      license='MIT',  classifiers=['Development Status :: 4 - Beta',
                                   'Intended Audience :: Developers']
      )



"""
Base Class for formatters. Does nothing with passed data by default;
Children should implement formatters as necessary
"""

# Import Built-Ins
import logging
from abc import ABCMeta, abstractmethod
import json
from functools import wraps
# Import Third-Party
import requests

# Import Homebrew

# Init Logging Facilities


log = logging.getLogger(__name__)


def return_api_response(formatter=None):
    """
    Decorator, which Applies the referenced formatter (if available) to the
    function output and adds it to the APIResponse Object's `formatted`
    attribute.
    :param formatter: ccapis.formatters.Formatter() obj
    :return: ccapis.api.response.RESTAPIResponse()
    """
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            print (func)
            print (args)
            print (kwargs)
            try:
                r = func(*args, **kwargs)
            except Exception:
                log.exception("return_api_response(): Error during call to %s(%s, %s)",
                              func.__name__, args, kwargs)
                raise

            # Check Status
            try:
                r.raise_for_status()
            except requests.HTTPError:
                log.exception("return_api_response: HTTPError for url %s",
                              r.request.url)

            #  Verify json data
            try:
                data = r.json()
            except json.JSONDecodeError:
                log.error('return_api_response: Error while parsing json. '
                          'Request url was: %s, result is: '
                          '%s', r.request.url, r.text)
                data = None
            except Exception:
                log.exception("return_api_response(): Unexpected error while parsing "
                              "json from %s", r.request.url)
                raise

            # Format, if available
            if formatter is not None and data:
                try:
                    r.formatted = formatter(data, *args, **kwargs)
                except Exception:
                    log.exception("Error while applying formatter!")

            return r

        return wrapper
    return decorator



class Formatter:
    """
    ABC Class to provide formatters for `bitex.utils.return_api_response()`.
    """
    def __init__(self):
        pass

    @staticmethod
    def format_pair(input_pair):
        """
        Returns the pair properly formatted for the exchange's API.
        :param input_pair: str
        :return: str
        """
        return input_pair

    @staticmethod
    def ticker(data, *args, **kwargs):
        """
        Returns list of ticker data in following format:
            [bid_price, ask_price, high, low, open, close, last, 24h_vol, ts]
        :param data: requests.response() obj
        :param args:
            args[0] 1st argument of ticker, interface object,
            args[1] 2nd argument of ticker, base,
            args[2] 3rd argument of ticker, counter
        :param kwargs:
        :return: list
        """
        return data

    @staticmethod
    def order_book(data, *args, **kwargs):
        """
        Returns dict of lists of lists of quotes in format [ts, price, size]
        ex.:
            {'bids': [['1480941692', '0.014', '10'],
                      ['1480941690', '0.013', '0.66'],
                      ['1480941688', '0.012', '3']],
             'asks': [['1480941691', '0.015', '1'],
                      ['1480941650', '0.016', '0.67'],
                      ['1480941678', '0.017', '23']]}
        :param data: requests.response() obj
        :param args:
        :param kwargs:
        :return: dict
        """
        return data

    @staticmethod
    def trades(data, *args, **kwargs):
        """
        Returns list of trades in format [ts, price, size, side]
        ex.:
            [['1480941692', '0.014', '10', 'sell'],
            ['1480941690', '0.013', '0.66', 'buy'],
            ['1480941688', '0.012', '3', 'buy']]
        :param data: requests.response() obj
        :param args:
        :param kwargs:
        :return: list
        """
        return data

    @staticmethod
    def order(data, *args, **kwargs):
        """
        Returns the order id as str if successful, else ""
        :param data: requests.response() obj
        :param args:
        :param kwargs:
        :return: str
        """
        return data

    @staticmethod
    def order_status(data, *args, **kwargs):
        """
        Returns True if it exists, False if it doesn't exist
        :param data: requests.response() obj
        :param args:
        :param kwargs:
        :return: bool
        """
        return data

    @staticmethod
    def cancel(data, *args, **kwargs):
        """
        returns True if it was cancelled successfully, else False
        :param data: requests.response() obj
        :param args:
        :param kwargs:
        :return: bool
        """
        return data

    @staticmethod
    def balance(data, *args, **kwargs):
        """
        Returns dict of available balances, with currency names as keys - this ignores
        any amount already involved in a trade (i.e. margin)
        ex.:
            {'BTC': '12.04', 'LTC': '444.12'}
        :param data: requests.response() obj
        :param args:
        :param kwargs:
        :return: dict
        """
        return data

    @staticmethod
    def withdraw(data, *args, **kwargs):
        """
        Returns a list giving details of success and transaction details, or failure
        and reason thererof
        ex.:
            [True, currency, amount, target_address, txid]
            [False, 'Reason for failure/ error message']
        :param data: requests.response() obj
        :param args:
        :param kwargs:
        :return: list
        """
        return data

    @staticmethod
    def deposit(data, *args, **kwargs):
        """
        Returns deposit address as str
        :param data: requests.response() obj
        :param args:
        :param kwargs:
        :return: str
        """
        return data



# Import Built-ins
import logging

# Import Third-Party

# Import Homebrew
from ccapis.formatters.base import Formatter


log = logging.getLogger(__name__)


class PoloniexFormatter(Formatter):

    @staticmethod
    def ticker(data, *args, **kwargs):
        # args   ticker(self, base, couter) args[0] == self(Poloniex object), args[1] == base, args[2] == counter
        data = data[args[0].get_pair(args[1], args[2])]
        return (data['highestBid'], data['lowestAsk'], None, None, None, None, data['last'], None, None)

    @staticmethod
    def order(data, *args, **kwargs):
        try:
            return data['orderNumber']
        except KeyError:
            return False

    @staticmethod
    def cancel(data, *args, **kwargs):
        return True if data['success'] else False



# Import Built-ins
import logging

# Import Third-Party

# Import Homebrew
from ccapis.formatters.base import Formatter

log = logging.getLogger(__name__)


class CoinoneFormatter(Formatter):

    @staticmethod
    def ticker(data, *args, **kwargs):
        # [bid_price, ask_price, high, low, open, close, last, 24h_vol, ts]
        return (None, None, data['high'], data['low'],
                data['first'], data['last'],data['last'], data['volume'], data['timestamp'])

    # @staticmethod
    # def order(data, *args, **kwargs):
    #     if data['status'] == '0000':
    #         return data['order_id']
    #     else:
    #         return False
    #
    # @staticmethod
    # def order_book(data, *args, **kwargs):
    #     if data['status'] == '0000':
    #         return data['data']
    #     else:
    #         return None
    #
    # @staticmethod
    # def cancel(data, *args, **kwargs):
    #     return True if data['status'] == '0000' else False



"""
Contains all API Client sub-classes, which store exchange specific details
and feature the respective exchanges authentication method (sign()).
"""
# Import Built-ins
import sys
import time
import math
import base64
import logging
import hashlib
import hmac
import urllib

# Import Homebrew
from ccapis.apis.rest import RESTAPIClient


log = logging.getLogger(__name__)


class BithumbREST(RESTAPIClient):
    def __init__(self, user_id='', key=None, secret=None, api_version=None,
                 url='https://api.bithumb.com', timeout=5):
        self.id = user_id
        super(BithumbREST, self).__init__(url, api_version=api_version,
                                           key=key, secret=secret,
                                           timeout=timeout)

    def sign(self, url, endpoint, endpoint_path, method_verb, *args, **kwargs):
        __endpoint = {
            'endpoint': endpoint
        }
        try:
            __uri = dict(endpoint, **kwargs['params'])
        except KeyError:
            __uri = endpoint

        __nonce = self.nonce()
        data = endpoint+ chr(0) + urllib.parse.urlencode(__uri) + chr(0) + __nonce
        data = data.encode('utf-8')

        h = hmac.new(bytes(self.secret.encode('utf-8')), data, hashlib.sha512)
        signature = base64.b64encode( h.hexdigest().encode('utf-8') ).decode('utf-8')

        headers = {
            'Api-Key': self.key,
            'Api-Sign': signature,
            'Api-Nonce': __nonce
        }

        return url , {'headers': headers}

    def public_query(self, endpoint, **kwargs):
        return self.query('GET', 'public/' + endpoint, **kwargs)

    def private_query(self, endpoint, **kwargs):
        return self.query('GET', endpoint, authenticate=True, **kwargs)

    # endpoint_item_array = {
    #     "endpoint": endpoint
    # };
    #
    # uri_array = dict(endpoint_item_array, **rgParams);  # Concatenate the two arrays.
    #
    # str_data = urllib.parse.urlencode(uri_array);
    # data = endpoint + chr(0) + str_data + chr(0) + nonce;
    # utf8_data = data.encode('utf-8');
    #
    # key = self.api_secret;
    # utf8_key = key.encode('utf-8');
    #
    # h = hmac.new(bytes(utf8_key), utf8_data, hashlib.sha512);
    # hex_output = h.hexdigest();
    # utf8_hex_output = hex_output.encode('utf-8');
    #
    # api_sign = base64.b64encode(utf8_hex_output);
    # utf8_api_sign = api_sign.decode('utf-8');

#curl_handle.setopt(curl_handle.HTTPHEADER, ['Api-Key: ' + self.api_key, 'Api-Sign: ' + utf8_api_sign, 'Api-Nonce: ' + nonce]);
